//
//  IPController.m
//  iProcrastinate
//
//  Created by Craig Otis on 6/17/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "IPController.h"
#import <DiscRecording/DiscRecording.h>
#import <DiscRecordingUI/DiscRecordingUI.h>


@implementation IPController

- (void) awakeFromNib {	
	// set the new custom cell
	NSTableColumn* column = [[filesTableView tableColumns] objectAtIndex:0];
	
	ImageTextCell* cell = [[[ImageTextCell alloc] init] autorelease];	
	[column setDataCell: cell];		

	[cell setDataDelegate: self];	
	
	//for subject view
	NSTableColumn* column2 = [[subjectTableView tableColumns] objectAtIndex:0];
	
	SubjectTextCell *cell2 = [[[SubjectTextCell alloc] init] autorelease];
	[column2 setDataCell:cell2];
	[cell2 setDataDelegate: self];
	
	NSLog(@"Cell columns set");
	
	[taskController setFilterPredicate:[NSPredicate predicateWithFormat:@"archiveState == 0"]];
	[archiveController setFilterPredicate:[NSPredicate predicateWithFormat:@"archiveState == 1"]];
}

- (NSArrayController *)filesController {
	return filesController;
}

- (void)acceptFilenameDrag:(NSString *)filename {
	NSManagedObjectContext *context = [[[NSDocumentController sharedDocumentController] currentDocument] managedObjectContext];
	NSManagedObject *myObject = nil;
	
	myObject = [NSEntityDescription insertNewObjectForEntityForName: @"File"
											 inManagedObjectContext: context];
	
	[myObject setValue: filename forKey: @"filePath"];
	NSLog(@"%@", filename);
	[filesController addObject:myObject];
	NSLog(@"FilesController has %d objects", [[filesController arrangedObjects] count]);
}

- (IBAction)beginAddingSubject:(id)sender {
	[subjectTitleField setStringValue:@""];
	[addInfoField setStringValue:@""];
	[subjectColor setColor:[NSColor blueColor]];
	
	[NSApp beginSheet:subjectSheet modalForWindow:mainWindow
		modalDelegate:self didEndSelector:NULL contextInfo:nil];	
}

- (IBAction)cancelAddingSubject:(id)sender {
	[subjectSheet orderOut:nil];
    [NSApp endSheet:subjectSheet];
}

- (IBAction)addSubject:(id)sender {
	
	NSManagedObjectContext * context = [[[NSDocumentController sharedDocumentController] currentDocument] managedObjectContext];
	NSManagedObject        * myObject  = nil;
	
    
	myObject = [NSEntityDescription insertNewObjectForEntityForName: @"Subject"
											 inManagedObjectContext: context];
	if ([[subjectTitleField stringValue] isEqualToString:@""])
		[myObject setValue:@"New Subject" forKey:@"title"];
	else
		[myObject setValue:[subjectTitleField stringValue] forKey:@"title"];
	[myObject setValue:[addInfoField stringValue] forKey:@"note"];
	
	NSData* colorData = [NSArchiver archivedDataWithRootObject:[subjectColor color]];
	NSLog(@"Setting subject's color to: %@", [subjectColor color]);
	[myObject setValue:colorData forKey:@"color"];
	
	[subjectController addObject:myObject];
	
	[subjectSheet orderOut:nil];
    [NSApp endSheet:subjectSheet];
}

- (IBAction)removeSubject:(id)sender {
	[subjectController removeObjectAtArrangedObjectIndex:[subjectTableView selectedRow]];
}

- (IBAction)addTask:(id)sender {
	NSLog(@"%d", [[taskController arrangedObjects] count]);
	NSManagedObjectContext * context = [[[NSDocumentController sharedDocumentController] currentDocument] managedObjectContext];
	NSManagedObject        * myObject  = nil;
    
	myObject = [NSEntityDescription insertNewObjectForEntityForName: @"Task"
											 inManagedObjectContext: context];
	
	[myObject setValue:@"New Task..." forKey:@"title"];
	[myObject setValue:[[NSCalendarDate calendarDate] dateByAddingYears:0
																 months:0
																   days:7
																  hours:0
																minutes:0
																seconds:0] forKey:@"date"];
	[myObject setValue: [[subjectController arrangedObjects] objectAtIndex:[subjectTableView selectedRow]] forKey:@"parentSubject"];
	
	NSEvent *theEvent;
	
	[taskTableView editColumn:0 row:[taskTableView selectedRow] withEvent:theEvent select:YES];
	
	//[taskController addObject:myObject];
	NSLog(@"%d", [[taskController arrangedObjects] count]);
	NSLog(@"Task Added");
}

- (IBAction)removeTask:(id)sender {
	[taskController removeObjectAtArrangedObjectIndex:[taskTableView selectedRow]];
}

- (IBAction)addStep:(id)sender {
	NSManagedObjectContext * context = [[[NSDocumentController sharedDocumentController] currentDocument] managedObjectContext];
	NSManagedObject        * myObject  = nil;
    
	myObject = [NSEntityDescription insertNewObjectForEntityForName: @"Step"
											 inManagedObjectContext: context];
	
	[myObject setValue:[[NSCalendarDate calendarDate] dateByAddingYears:0
																 months:0
																   days:3
																  hours:0
																minutes:0
																seconds:0] forKey:@"date"];
	[myObject setValue: [[taskController arrangedObjects] objectAtIndex:[taskTableView selectedRow]] forKey:@"parentTask"];
	
	NSLog(@"Task Added");
}

- (IBAction)removeStep:(id)sender {
	[stepController removeObjectAtArrangedObjectIndex:[stepTableView selectedRow]];
}

- (IBAction)customOpenHelp:(id)sender {
	NSLog(@"Working");
	[helpPanel makeKeyAndOrderFront:nil];
}

- (IBAction)archiveTask:(id)sender {
	NSLog(@"Archiving");
	NSLog(@"%@", [[taskController arrangedObjects] objectAtIndex:[taskTableView selectedRow]]);
	int theRow = [taskTableView selectedRow];
	[[[taskController arrangedObjects] objectAtIndex:theRow] setValue:[NSNumber numberWithBool:YES] forKey:@"archiveState"];
	NSLog(@"%@", [[taskController arrangedObjects] objectAtIndex:[taskTableView selectedRow]]);
	[taskController rearrangeObjects];
	[taskTableView selectRow:theRow byExtendingSelection:NO];
}

- (IBAction)restoreArchivedTask:(id)sender {
	[[[archiveController arrangedObjects] objectAtIndex:[archiveTableView selectedRow]] setValue:[NSNumber numberWithBool:NO] forKey:@"archiveState"];
	[taskController rearrangeObjects];
}

- (IBAction)showAllSubjects:(id)sender {
	[subjectTableView selectAll:self];
}

- (IBAction)burnFilesToCd:(id)sender {
	int numFiles = [[filesController arrangedObjects] count];
	NSLog(@"Initiating burn for %d files", numFiles);
	NSCalendarDate *now = [NSCalendarDate calendarDate];
	NSString *datestr = [now descriptionWithCalendarFormat:@"%m/%d/%y"];
	DRFolder *mainFolder = [[DRFolder alloc] initWithName:[NSString stringWithFormat:@"iProcrastinate Files %@", datestr]];
	DRFolder *myFolder = nil;
	int i;
	for (i = 0; i < numFiles; i++) {
		NSString *filename = [[[filesController arrangedObjects] objectAtIndex:i] valueForKey:@"filePath"];
		myFolder = [[DRFile alloc] initWithPath:filename];
		[mainFolder addChild:myFolder];
	}
	
	DRTrack *myTrack = [DRTrack trackForRootFolder:mainFolder];
	if (myTrack)
	{
	DRBurnSetupPanel*	bsp = [DRBurnSetupPanel setupPanel];
	
	// We'll be the delegate for the setup panel. This allows us to show off some 
	// of the customization you can do.
	[bsp setDelegate:self];
	
	if ([bsp runSetupPanel] == NSOKButton)
	{
		DRBurnProgressPanel*	bpp = [DRBurnProgressPanel progressPanel];
		
		[bpp setDelegate:self];
		
		// And start off the burn itself. This will put up the progress dialog 
		// and do all the nice pretty things that a happy app does.
		//[bpp beginProgressPanelForBurn:[bsp burnObject] layout:myTrack];
		
		//If you wanted to run this as a sheet you would have sent
		[bpp beginProgressSheetForBurn:[bsp burnObject] layout:myTrack modalForWindow:mainWindow];
		
	}
	else
		[bsp close];
	}
}

- (void) burnProgressPanelWillBegin:(NSNotification*)aNotification
{
	burning = YES;	// Keep the app from being quit from underneath the burn.
}

- (void) burnProgressPanelDidFinish:(NSNotification*)aNotification
{
	burning = NO;	// OK we can quit now.
}

- (BOOL) setupPanel:(DRSetupPanel*)aPanel deviceCouldBeTarget:(DRDevice*)device
{
#if 0
	// This bit of code shows how to filter devices bases on the properties of the device
	// For example, it's possible to limit the drives displayed to only those hooked up over
	// firewire, or converesely, you could NOT show drives if there was some reason to. 
	NSDictionary*	deviceInfo = [device info];
	if ([[deviceStatus objectForKey:DRDevicePhysicalInterconnectKey] isEqualToString:DRDevicePhysicalInterconnectFireWire])
		return YES;
	else
		return NO;
#else
	return YES;
#endif
}

- (BOOL) burnProgressPanel:(DRBurnProgressPanel*)theBurnPanel burnDidFinish:(DRBurn*)burn
{
	NSDictionary*	burnStatus = [burn status];
	NSString*		state = [burnStatus objectForKey:DRStatusStateKey];
	
	if ([state isEqualToString:DRStatusStateFailed])
	{
		NSDictionary*	errorStatus = [burnStatus objectForKey:DRErrorStatusKey];
		NSString*		errorString = [errorStatus objectForKey:DRErrorStatusErrorInfoStringKey];
		
		NSLog(@"The burn failed (%@)!", errorString);
	}
	else
		NSLog(@"Burn finished fine");
	
	return YES;
}

- (IBAction)addFile:(id)sender {
	NSOpenPanel*		openPanel = [NSOpenPanel openPanel];
	
	// ask the user for the list of files to burn. 
	[openPanel setAllowsMultipleSelection:YES];
	[openPanel setCanChooseDirectories:YES];
	[openPanel setCanChooseFiles:YES];
	[openPanel setResolvesAliases:YES];
	[openPanel setDelegate:self];
	[openPanel setTitle:@"Select a file to add."];
	[openPanel setPrompt:@"Select"];
	
	if ([openPanel runModalForTypes:nil] == NSOKButton)
	{
		int files = [[openPanel filenames] count];
		NSLog(@"Attempting to add %d files", files);
		int j;
		for (j = 0; j < files; j++) {
			[self acceptFilenameDrag:[[openPanel filenames] objectAtIndex:j]];
		}
	}
}

- (IBAction)removeThisFile:(id)sender {
	[filesController removeObjectAtArrangedObjectIndex:[filesTableView selectedRow]];
}

- (IBAction)noteStepDidChange:(id)sender {
	int selected = [noteStep selectedSegment];
	if (selected == 0) {
		[stepDatePickerButton setHidden:YES];
		[addStepButton setHidden:YES];
		[removeStepButton setHidden:YES];
		[extraSpace setHidden:YES];
		[stepsTableView setHidden:YES];
		[filesScrollView setHidden:NO];
		[notesTextField setHidden:NO];
		[burnButton setHidden:NO];
		[addFileButton setHidden:NO];
		[removeFileButton setHidden:NO];
		NSLog(@"0");
	}	else {
		[burnButton setHidden:YES];
		[addFileButton setHidden:YES];
		[removeFileButton setHidden:YES];
		[filesScrollView setHidden:YES];
		[notesTextField setHidden:YES];
		[addStepButton setHidden:NO];
		[removeStepButton setHidden:NO];
		[extraSpace setHidden:NO];
		[stepDatePickerButton setHidden:NO];
		[stepsTableView setHidden:NO];
		NSLog(@"1");
	}
}

@end
