//
//  SubjectTextCell.m
//  iProcrastinate
//
//  Created by Craig Otis on 6/18/07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#import "SubjectTextCell.h"
#import "IPController.h"
#import "CTGradient.h"

@implementation SubjectTextCell
- (void)dealloc {
	[self setDataDelegate: nil];
	[self setIconKeyPath: nil];
	[self setPrimaryTextKeyPath: nil];
	[self setSecondaryTextKeyPath: nil];
    [super dealloc];
}

- copyWithZone:(NSZone *)zone {
	SubjectTextCell *cell = (SubjectTextCell *)[super copyWithZone:zone];
	cell->delegate = nil;
	[cell setDataDelegate: delegate];
    return cell;
}

- (void) setIconKeyPath: (NSString*) path {
	[iconKeyPath autorelease];
	iconKeyPath = [path retain];
}
- (void) setPrimaryTextKeyPath: (NSString*) path {
	[primaryTextKeyPath autorelease];
	primaryTextKeyPath = [path retain];	
}
- (void) setSecondaryTextKeyPath: (NSString*) path {
	[secondaryTextKeyPath autorelease];
	secondaryTextKeyPath = [path retain];	
}

- (void) setDataDelegate: (NSObject*) aDelegate {
	[aDelegate retain];	
	[delegate autorelease];
	delegate = aDelegate;	
}

- (id) dataDelegate {
	if (delegate) return delegate;
	return self; // in case there is no delegate we try to resolve values by using key paths
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
	[self setTextColor:[NSColor blackColor]];

	NSManagedObjectContext *context = [[[NSDocumentController sharedDocumentController] currentDocument] managedObjectContext];
	NSEntityDescription *entityDescription = [NSEntityDescription entityForName:@"Subject" inManagedObjectContext:context];
	NSFetchRequest *request = [[[NSFetchRequest alloc] init] autorelease];
	[request setEntity:entityDescription];
	NSString* data = [self objectValue];
	NSLog(@"Data is %@", [data description]);
	NSPredicate *predicate = [NSPredicate predicateWithFormat:@"title == %@", data];
	[request setPredicate:predicate];
	NSError *error = nil;
	NSArray *array = [context executeFetchRequest:request error:&error];
	
	NSManagedObject *currentObject = [array objectAtIndex:0];
	
	// give the delegate a chance to set a different data object
	//if ([[self dataDelegate] respondsToSelector: @selector(dataElementForCell:)]) {
	//	data = [[self dataDelegate] dataElementForCell:self];
	//}
	
	//TODO: Selection with gradient and selection color in white with shadow
	// check out http://www.cocoadev.com/index.pl?NSTableView
	
	BOOL elementDisabled    = NO;	
	if ([[self dataDelegate] respondsToSelector: @selector(disabledForCell:data:)]) {
		elementDisabled = [[self dataDelegate] disabledForCell: self data: data];
	}
	NSNumber *fsize;
	NSFileManager *fileManager = [NSFileManager defaultManager];
	NSDictionary *fattrs = [fileManager fileAttributesAtPath:data traverseLink:NO];
	fsize = [fattrs objectForKey:NSFileSize];
	NSString *ftype = [fattrs objectForKey:NSFileType];
	int humanSize = ([fsize intValue] / 1024);
	
	NSColor* primaryColor   = [self isHighlighted] ? [NSColor alternateSelectedControlTextColor] : (elementDisabled? [NSColor disabledControlTextColor] : [NSColor textColor]);
	NSString* primaryText   = data; //[pathArray objectAtIndex:([pathArray count]-1)];
	
	NSRect bounds = NSMakeRect(cellFrame.origin.x -1, cellFrame.origin.y -1, cellFrame.size.width + 3, cellFrame.size.height + 2);
	if ([self isHighlighted]) {
		CTGradient *newGradient = [CTGradient gradientWithBeginningColor:[NSColor colorWithCalibratedRed:.549 green:.596 blue:.690 alpha:1.0] endingColor:[NSColor colorWithCalibratedRed:.678 green:.733 blue:.816 alpha:1.0]];
		[newGradient fillRect:bounds angle:270];
	}

//NSLog(@"App has focus: %@", [NSApp l
	
	NSDictionary* primaryTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys: primaryColor, NSForegroundColorAttributeName,
		[NSFont systemFontOfSize:13], NSFontAttributeName, nil];	
	[primaryText drawAtPoint:NSMakePoint(cellFrame.origin.x + 40, cellFrame.origin.y + 8) withAttributes:primaryTextAttributes];
	
		
	NSColor* secondaryColor = [self isHighlighted] ? [NSColor alternateSelectedControlTextColor] : [NSColor disabledControlTextColor];
	
	NSString *sizeString;
	if ([ftype isEqualToString:@"NSFileTypeDirectory"])
		sizeString = @"Directory";
	else
		if (humanSize < 1024)
			sizeString = [NSString stringWithFormat:@"%d KB", humanSize];
	else
		if (humanSize > 1024 && humanSize < (1024 * 1024))
			sizeString = [NSString stringWithFormat:@"%d MB", (humanSize / 1024)];
	else
		sizeString = [NSString stringWithFormat:@"%d GB", (humanSize / (1024 * 1024))];
	
	
	NSString* secondaryText = [currentObject valueForKey:@"note"];
	
	NSDictionary* secondaryTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys: secondaryColor, NSForegroundColorAttributeName,
		[NSFont systemFontOfSize:10], NSFontAttributeName, nil];
	NSLog(@"%d", cellFrame.origin.y);
	[secondaryText drawAtPoint:NSMakePoint(cellFrame.origin.x + 48, cellFrame.origin.y + 28) withAttributes:secondaryTextAttributes];
	
	
	[[NSGraphicsContext currentContext] saveGraphicsState];
/*
	float yOffset = cellFrame.origin.y;

	if ([controlView isFlipped]) {
		NSAffineTransform* xform = [NSAffineTransform transform];
		[xform translateXBy:0.0 yBy: cellFrame.size.height];
		[xform scaleXBy:1.0 yBy:-1.0];
		[xform concat];		
		yOffset = 0-cellFrame.origin.y;
	}
 */
	NSRect borderRect2 = NSMakeRect(7,cellFrame.origin.y+(cellFrame.size.height/2) - 13,26,26);
	NSLog(@"%d",cellFrame.origin.y);
	NSRect borderRect = NSMakeRect(8,cellFrame.origin.y+cellFrame.size.height/2 - 12,24,24);
	NSRect myRect = NSMakeRect(10,cellFrame.origin.y+cellFrame.size.height/2 - 10,20,20);
	NSLog(@"%@", [NSUnarchiver unarchiveObjectWithData:[currentObject valueForKey:@"color"]]);
	
	[[NSColor blackColor] set];
	
	NSBezierPath *colorPath = [NSBezierPath bezierPathWithRect:borderRect2];
	[colorPath fill];
	
	[[NSColor whiteColor] set];
	colorPath = [NSBezierPath bezierPathWithRect:borderRect];
	[colorPath fill];

	[(NSColor *)[NSUnarchiver unarchiveObjectWithData:[currentObject valueForKey:@"color"]] set];
	colorPath = [NSBezierPath bezierPathWithRect:myRect];
	[colorPath fill];

	NSImageInterpolation interpolation = [[NSGraphicsContext currentContext] imageInterpolation];
	
	[[NSGraphicsContext currentContext] setImageInterpolation: NSImageInterpolationHigh];	
	[[NSGraphicsContext currentContext] setImageInterpolation: interpolation];
	
	[[NSGraphicsContext currentContext] restoreGraphicsState];	
}

#pragma mark - 
#pragma mark Delegate methods

- (NSImage*) iconForCell: (ImageTextCell*) cell data: (NSObject*) data {
	if (iconKeyPath) {
		return [NSImage imageNamed:@"Woof.gif"];
	}
	return nil;
}
- (NSString*) primaryTextForCell: (ImageTextCell*) cell data: (NSObject*) data {
	if (primaryTextKeyPath) {
		return @"Primary";
	}
	return nil;	
}
- (NSString*) secondaryTextForCell: (ImageTextCell*) cell data: (NSObject*) data {
	if (primaryTextKeyPath) {
		return @"Secondary";
	}
	return nil;		
}

@end
